"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ink_1 = require("ink");
var lodash_1 = require("lodash");
var react_1 = __importDefault(require("react"));
var ProgressBar_1 = require("./ProgressBar");
var Results_1 = require("./Results");
var Run_1 = require("./Run");
var getStepsCounts = function (runs) {
    var completeCount = 0;
    var totalCount = 0;
    runs.forEach(function (run) {
        completeCount += run.steps.filter(function (step) { return step.status !== "runs" && step.status !== "queued"; }).length;
        totalCount += run.steps.length;
    });
    return { completeCount: completeCount, totalCount: totalCount };
};
var renderRuns = function (_a) {
    var runs = _a.runs, showSteps = _a.showSteps;
    return runs.map(function (run) { return (react_1.default.createElement(Run_1.Run, { key: run.name, showSteps: !!showSteps, run: run })); });
};
var Runs = function (_a) {
    var summary = _a.summary, runs = _a.runs;
    var completeRuns = runs.filter(function (run) {
        return run.status === "pass" || run.status === "fail";
    });
    var runsRuns = runs.filter(function (run) { return run.status === "runs"; });
    var failRuns = runs.filter(function (run) { return run.status === "fail"; });
    var completeRunsHtml = renderRuns({
        runs: completeRuns
    });
    var runsRunsHtml = renderRuns({
        runs: runsRuns,
        showSteps: true
    });
    var failRunsHtml = summary
        ? renderRuns({ runs: failRuns, showSteps: true })
        : null;
    var actionText = summary ? "ran" : "is running";
    var startTime = lodash_1.min(runs.map(function (run) { return run.startTime; }));
    return (react_1.default.createElement(ink_1.Box, { flexDirection: "column" },
        react_1.default.createElement(ink_1.Static, null, completeRunsHtml),
        react_1.default.createElement(ink_1.Color, { bold: true, cyan: true }, "\n\uD83D\uDC3A QA Wolf " + actionText + " your tests!\n"),
        react_1.default.createElement(ink_1.Box, { flexDirection: "column" }, runsRunsHtml),
        !summary && react_1.default.createElement(ProgressBar_1.ProgressBar, __assign({}, getStepsCounts(runs))),
        !!summary && !!summary.fail && (react_1.default.createElement(ink_1.Color, { bold: true, red: true }, "\n" + summary.fail + " failed")),
        failRunsHtml,
        react_1.default.createElement(Results_1.Results, { startTime: startTime, summary: summary })));
};
exports.default = (function (props) {
    ink_1.render(react_1.default.createElement(Runs, __assign({}, props)));
});
//# sourceMappingURL=Runs.js.map