"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findHref = function (events) {
    return events[0].data.href;
};
exports.orderEventsByTime = function (events) {
    var orderedEvents = [];
    for (var _i = 0, events_1 = events; _i < events_1.length; _i++) {
        var originalEvent = events_1[_i];
        var event_1 = JSON.parse(JSON.stringify(originalEvent));
        var positions = (event_1.data && event_1.data.positions) || [];
        if (positions.length) {
            var firstOffset = positions[0].timeOffset;
            event_1.timestamp += firstOffset;
            for (var _a = 0, positions_1 = positions; _a < positions_1.length; _a++) {
                var position = positions_1[_a];
                position.timeOffset -= firstOffset;
            }
        }
        orderedEvents.push(event_1);
    }
    orderedEvents.sort(function (a, b) { return a.timestamp - b.timestamp; });
    orderedEvents.forEach(function (event, index) { return (event.id = index); });
    return orderedEvents;
};
exports.isMouseDownEvent = function (event) {
    if (!event || !event.data)
        return false;
    var data = event.data;
    return !!(data.source === 2 && data.type === 1 && data.isTrusted);
};
exports.isTypeEvent = function (event) {
    if (!event || !event.data)
        return false;
    var data = event.data;
    return !!(data.source === 5 && data.isTrusted && data.text);
};
exports.planClickActions = function (events) {
    var steps = [];
    for (var _i = 0, events_2 = events; _i < events_2.length; _i++) {
        var event_2 = events_2[_i];
        if (!exports.isMouseDownEvent(event_2))
            continue;
        steps.push({
            locator: event_2.data.properties,
            pageId: event_2.pageId,
            sourceEventId: event_2.id,
            type: "click"
        });
    }
    return steps;
};
exports.planJob = function (originalEvents) {
    var url = exports.findHref(originalEvents);
    var events = exports.orderEventsByTime(originalEvents);
    var steps = exports.planClickActions(events).concat(exports.planTypeActions(events));
    steps.sort(function (a, b) { return a.sourceEventId - b.sourceEventId; });
    var job = { name: "job", steps: steps, url: url };
    return job;
};
exports.planTypeActions = function (events) {
    var steps = [];
    var lastXpath = null;
    for (var i = events.length - 1; i >= 0; i--) {
        var event_3 = events[i];
        if (!exports.isTypeEvent(event_3))
            continue;
        if (event_3.data.xpath === lastXpath)
            continue;
        steps.push({
            locator: event_3.data.properties,
            pageId: event_3.pageId,
            sourceEventId: event_3.id,
            type: "type",
            value: event_3.data.text
        });
        lastXpath = event_3.data.xpath;
    }
    return steps;
};
//# sourceMappingURL=planner.js.map