(function () {
  'use strict';

  var click = function (element) {
      var clickableElement = findClickableAncestor(element);
      var eventOptions = { bubbles: true };
      clickableElement.dispatchEvent(new MouseEvent("mousedown", eventOptions));
      clickableElement.dispatchEvent(new MouseEvent("mouseup", eventOptions));
      clickableElement.dispatchEvent(new MouseEvent("click", eventOptions));
  };
  var findClickableAncestor = function (element) {
      if (element.click)
          return element;
      if (!element.parentElement) {
          throw "Element " + element + " does not have parent";
      }
      return findClickableAncestor(element.parentElement);
  };
  var setInputValue = function (element, value) {
      if (value === void 0) { value = null; }
      element.value = value || "";
  };

  var actions = /*#__PURE__*/Object.freeze({
    click: click,
    setInputValue: setInputValue
  });

  var findElementByXpath = function (xpath) {
      var result = document.evaluate(xpath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
      if (result.singleNodeValue) {
          return result.singleNodeValue;
      }
      return null;
  };
  var buildXpath = function (element) {
      if (!element || element.nodeType !== 1)
          return "";
      if (element.id)
          return "//*[@id='" + element.id + "']";
      var children = element.parentNode ? element.parentNode.children : [];
      var sames = [].filter.call(children, function (x) {
          return x.tagName === element.tagName;
      });
      var result = buildXpath(element.parentNode) +
          "/" +
          element.tagName.toLowerCase() +
          (sames.length > 1
              ? "[" + ([].indexOf.call(sames, element) + 1) + "]"
              : "");
      return result;
  };
  var getXpath = function (element) {
      var result = buildXpath(element);
      return result
          .replace("svg", "*[name()='svg']")
          .replace("path", "*[name()='path']");
  };
  var compareXpaths = function (xpathA, xpathB) {
      if (!xpathA || !xpathB)
          return false;
      if (xpathA === xpathB)
          return true;
      var resolveSameElement = findElementByXpath(xpathA) === findElementByXpath(xpathB);
      return resolveSameElement;
  };

  var xpath = /*#__PURE__*/Object.freeze({
    findElementByXpath: findElementByXpath,
    getXpath: getXpath,
    compareXpaths: compareXpaths
  });

  var getLabels = function (element) {
      var labelElements = element.labels;
      if (!labelElements || !labelElements.length)
          return null;
      var labels = [];
      for (var i = 0; i < labelElements.length; i++) {
          var textContent = labelElements[i].textContent;
          if (textContent) {
              labels.push(textContent.toLowerCase());
          }
      }
      return labels;
  };
  var getParentText = function (element) {
      if (!element.parentElement)
          return null;
      if (element.parentElement.textContent) {
          var parentText = [];
          for (var i = 0; i < element.parentElement.children.length; i++) {
              var textContent = element.parentElement.children[i].textContent;
              if (textContent) {
                  parentText.push(textContent.toLowerCase());
              }
              parentText.push(element.parentElement.textContent.toLowerCase());
          }
          return parentText;
      }
      return getParentText(element.parentElement);
  };
  var getPlaceholder = function (element) {
      if (!element.placeholder)
          return null;
      return element.placeholder.toLowerCase();
  };
  var getLocator = function (element) {
      if (!element) {
          return null;
      }
      return {
          classList: (element.className || "").length
              ? element.className.split(" ")
              : null,
          href: element.href || null,
          id: element.id || null,
          inputType: element.type || null,
          labels: getLabels(element),
          name: element.name || null,
          parentText: getParentText(element),
          placeholder: getPlaceholder(element),
          tagName: element.tagName ? element.tagName.toLowerCase() : null,
          textContent: getTextContent(element),
          xpath: getXpath(element)
      };
  };
  var getTextContent = function (element) {
      if (!element.textContent)
          return null;
      return element.textContent.toLowerCase();
  };

  var locator = /*#__PURE__*/Object.freeze({
    getLabels: getLabels,
    getParentText: getParentText,
    getPlaceholder: getPlaceholder,
    getLocator: getLocator,
    getTextContent: getTextContent
  });

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */

  function __awaiter(thisArg, _arguments, P, generator) {
      return new (P || (P = Promise))(function (resolve, reject) {
          function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
          function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
          function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
          step((generator = generator.apply(thisArg, _arguments || [])).next());
      });
  }

  function __generator(thisArg, body) {
      var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
      return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
      function verb(n) { return function (v) { return step([n, v]); }; }
      function step(op) {
          if (f) throw new TypeError("Generator is already executing.");
          while (_) try {
              if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
              if (y = 0, t) op = [op[0] & 2, t.value];
              switch (op[0]) {
                  case 0: case 1: t = op; break;
                  case 4: _.label++; return { value: op[1], done: false };
                  case 5: _.label++; y = op[1]; op = [0]; continue;
                  case 7: op = _.ops.pop(); _.trys.pop(); continue;
                  default:
                      if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                      if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                      if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                      if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                      if (t[2]) _.ops.pop();
                      _.trys.pop(); continue;
              }
              op = body.call(thisArg, _);
          } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
          if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
      }
  }

  var computeArraySimilarityScore = function (compare, base) {
      if (!base || !compare || !base.length)
          return 0;
      var similarityScore = 0;
      base.forEach(function (value) {
          if (compare.includes(value)) {
              similarityScore += 1;
          }
      });
      return Math.round((similarityScore / base.length) * 100);
  };
  var computeStringSimilarityScore = function (compare, base) {
      if (!base || !compare)
          return 0;
      if (base === compare)
          return 100;
      return 0;
  };
  var computeSimilarityScore = function (compare, base) {
      var score = 0;
      score += computeArraySimilarityScore(compare.classList, base.classList);
      score += computeStringSimilarityScore(compare.href, base.href);
      score += computeStringSimilarityScore(compare.id, base.id);
      score += computeStringSimilarityScore(compare.inputType, base.inputType);
      score += computeArraySimilarityScore(compare.labels, base.labels);
      score += computeStringSimilarityScore(compare.name, base.name);
      score += computeArraySimilarityScore(compare.parentText, base.parentText);
      score += computeStringSimilarityScore(compare.placeholder, base.placeholder);
      score += computeStringSimilarityScore(compare.tagName, base.tagName);
      score += computeStringSimilarityScore(compare.textContent, base.textContent);
      score += compareXpaths(compare.xpath, base.xpath) ? 100 : 0;
      return score;
  };
  var computeMaxPossibleScore = function (base) {
      var score = 0;
      Object.values(base).forEach(function (value) {
          if (value !== null) {
              score += 100;
          }
      });
      if (!score) {
          throw "Base element " + base + " has no recorded properties";
      }
      return score;
  };

  var DEFAULT_THRESHOLD = 0.75;
  var computeSimilarityScores = function (step, elements) {
      var base = step.locator;
      if (!base) {
          throw "Action does not have associated locator";
      }
      var scores = [];
      for (var i = 0; i < elements.length; i++) {
          var compare = getLocator(elements[i]);
          if (!compare) {
              scores.push(0);
          }
          else {
              var score = computeSimilarityScore(compare, base);
              scores.push(score);
          }
      }
      return scores;
  };
  var findCandidateElements = function (step) {
      if (step.type === "type") {
          return document.getElementsByTagName("input");
      }
      return document.getElementsByTagName("*");
  };
  var findTopElement = function (step, threshold) {
      if (threshold === void 0) { threshold = DEFAULT_THRESHOLD; }
      var elements = findCandidateElements(step);
      var scores = computeSimilarityScores(step, elements);
      var maxScore = Math.max.apply(Math, scores);
      var maxPossibleScore = computeMaxPossibleScore(step.locator);
      if (maxScore / maxPossibleScore < threshold) {
          return null;
      }
      if (scores.filter(function (score) { return score === maxScore; }).length > 1) {
          return null;
      }
      var topMatch = elements[scores.indexOf(maxScore)];
      return topMatch;
  };
  var waitForElement = function (step, timeout, threshold) {
      if (timeout === void 0) { timeout = 5000; }
      if (threshold === void 0) { threshold = DEFAULT_THRESHOLD; }
      return __awaiter(void 0, void 0, Promise, function () {
          return __generator(this, function (_a) {
              return [2, new Promise(function (resolve, reject) {
                      var intervalId = setInterval(function () {
                          var element = findTopElement(step, threshold);
                          if (element) {
                              console.log("waitForElement: found element", element);
                              clearInterval(intervalId);
                              clearTimeout(rejectId);
                              resolve(element);
                          }
                      }, 100);
                      var rejectId = setTimeout(function () {
                          reject("No element found for step " + JSON.stringify(step));
                          clearInterval(intervalId);
                      }, timeout);
                  })];
          });
      });
  };

  var rank = /*#__PURE__*/Object.freeze({
    computeSimilarityScores: computeSimilarityScores,
    findCandidateElements: findCandidateElements,
    findTopElement: findTopElement,
    waitForElement: waitForElement
  });

  var runStep = function (step) { return __awaiter(void 0, void 0, void 0, function () {
      var element;
      return __generator(this, function (_a) {
          switch (_a.label) {
              case 0: return [4, waitForElement(step)];
              case 1:
                  element = (_a.sent());
                  if (step.type === "click") {
                      click(element);
                  }
                  else {
                      setInputValue(element, step.value);
                  }
                  return [2];
          }
      });
  }); };

  var qawolf = {
      actions: actions,
      locator: locator,
      rank: rank,
      runStep: runStep,
      xpath: xpath
  };
  if (typeof window !== "undefined" && !window.qawolf) {
      window.qawolf = qawolf;
      console.log("loaded qawolf");
  }

}());
