"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var xpath_1 = require("./xpath");
exports.getLabels = function (element) {
    var labelElements = element.labels;
    if (!labelElements || !labelElements.length)
        return null;
    var labels = [];
    for (var i = 0; i < labelElements.length; i++) {
        var textContent = labelElements[i].textContent;
        if (textContent) {
            labels.push(textContent.toLowerCase());
        }
    }
    return labels;
};
exports.getParentText = function (element) {
    if (!element.parentElement)
        return null;
    if (element.parentElement.textContent) {
        var parentText = [];
        for (var i = 0; i < element.parentElement.children.length; i++) {
            var textContent = element.parentElement.children[i].textContent;
            if (textContent) {
                parentText.push(textContent.toLowerCase());
            }
            parentText.push(element.parentElement.textContent.toLowerCase());
        }
        return parentText;
    }
    return exports.getParentText(element.parentElement);
};
exports.getPlaceholder = function (element) {
    if (!element.placeholder)
        return null;
    return element.placeholder.toLowerCase();
};
exports.getLocator = function (element) {
    if (!element) {
        return null;
    }
    return {
        classList: (element.className || "").length
            ? element.className.split(" ")
            : null,
        href: element.href || null,
        id: element.id || null,
        inputType: element.type || null,
        labels: exports.getLabels(element),
        name: element.name || null,
        parentText: exports.getParentText(element),
        placeholder: exports.getPlaceholder(element),
        tagName: element.tagName ? element.tagName.toLowerCase() : null,
        textContent: exports.getTextContent(element),
        xpath: xpath_1.getXpath(element)
    };
};
exports.getTextContent = function (element) {
    if (!element.textContent)
        return null;
    return element.textContent.toLowerCase();
};
//# sourceMappingURL=locator.js.map